<?php
// Own namespace
namespace Org\Mxchange\City\Helper\Login;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Factory\ObjectFactory;
use Org\Mxchange\CoreFramework\Registry\Registry;
use Org\Mxchange\CoreFramework\Request\Requestable;

/**
 * A helper for City to login. This login helper first checks what setting
 * (cookie or session) the admin has choosen then overwrites it with the setting
 * from current user. The registry instance should hold an instance of this user
 * class at key 'user' else an exception will be thrown. After this the setting
 * from a login form will be taken as login method and be stored in database
 * for later usage.
 *
 * The user shall be able to choose "Default login method" or similar to use his
 * own login method.
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2015, 2016 City Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class CityLoginHelper extends BaseLoginHelper implements HelpableLogin {
	/**
	 * The login method we shall choose
	 */
	private $authMethod = '';

	// Exception constants
	const EXCEPTION_INVALID_USER_INSTANCE = 0x190;

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this class by given request instance
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @return	$helperInstance		An instance of this helper class
	 * @throws	UserInstanceMissingException	If the user instance in registry
	 *											is missing or invalid
	 */
	public static final function createCityLoginHelper (Requestable $requestInstance) {
		// Get a new instance first
		$helperInstance = new CityLoginHelper();

		// Get a user instance from registry
		$userInstance = Registry::getRegistry()->getInstance('user');

		// Is this instance valid?
		if (!$userInstance instanceof ManageableAccount) {
			// Thrown an exception here
			throw new UserInstanceMissingException (array($helperInstance, 'user'), self::EXCEPTION_INVALID_USER_INSTANCE);
		} // END - if

		// Set default login method from config
		$helperInstance->setDefaultAuthMethod();

		// Set request instance
		$helperInstance->setRequestInstance($requestInstance);

		// Return the prepared instance
		return $helperInstance;
	}

	/**
	 * Setter for default login method from config
	 *
	 * @return	void
	 */
	protected function setDefaultAuthMethod () {
		$this->authMethod = $this->getConfigInstance()->getConfigEntry('auth_method_class');
	}

	/**
	 * Execute the login request by given response instance. This instance can
	 * be used for sending cookies or at least the session id out.
	 *
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 */
	public function executeLogin (Responseable $responseInstance) {
		// Get an instance from the login method
		$loginInstance = ObjectFactory::createObjectByName($this->authMethod, array($responseInstance));

		// Set user cookie
		$loginInstance->setUserAuth(FrameworkBootstrap::getRequestInstance()->getRequestElement('username'));

		// Set password cookie
		$loginInstance->setPasswordAuth(FrameworkBootstrap::getRequestInstance()->getRequestElement('pass_hash'));

		// Remember this login instance for later usage
		Registry::getRegistry()->addInstance('login', $loginInstance);
	}
}
